@@ -37,12 +37,6 @@
 #define ACTIVE_GH	\
 	(ghex_application_window_get_hex (self))
 
-#define GH_FOR_TAB(TV, NUM) \
-		(HEX_WIDGET(adw_tab_page_get_child (adw_tab_view_get_nth_page (TV, NUM))));
-
-#define TAB_FOR_GH(GH) \
-	(adw_tab_view_get_page (ADW_TAB_VIEW(self->hex_tab_view), GTK_WIDGET(GH)))
-
 /* Translators: this is the string for an untitled buffer that will
  * be displayed in the titlebar when a user does File->New
  */
@@ -176,6 +170,28 @@ static void ghex_application_window_connect_hex_signals (GHexApplicationWindow *
 
 /* GHexApplicationWindow -- PRIVATE FUNCTIONS */
 
+static HexWidget *
+get_gh_for_page (GHexApplicationWindow *self, AdwTabPage *page)
+{
+	GtkWidget *box = adw_tab_page_get_child (page);
+
+	for (GtkWidget *child = gtk_widget_get_first_child (box);
+			child != NULL;
+			child = gtk_widget_get_next_sibling (child))
+	{
+		if (HEX_IS_WIDGET (child))
+			return HEX_WIDGET(child);
+	}
+
+	return NULL;
+}
+
+static HexWidget *
+get_gh_for_tab (GHexApplicationWindow *self, AdwTabView *tv, int num)
+{
+	return get_gh_for_page (self, adw_tab_view_get_nth_page (tv, num));
+}
+
 /* Common macro to apply something to the 'gh' of each tab of the tab view.
  *
  * Between _START and _END, put in function calls that use `gh` to be applied
@@ -186,9 +202,8 @@ static void ghex_application_window_connect_hex_signals (GHexApplicationWindow *
 	AdwTabView *tab_view = ADW_TAB_VIEW(self->hex_tab_view);				\
 	int i;																	\
 	for (i = adw_tab_view_get_n_pages(tab_view) - 1; i >= 0; --i) {			\
-		HexWidget *gh;														\
-		gh = HEX_WIDGET(adw_tab_page_get_child (							\
-					adw_tab_view_get_nth_page (tab_view, i)));
+		AdwTabPage *tab_page = adw_tab_view_get_nth_page (tab_view, i);		\
+		HexWidget *gh = get_gh_for_tab (self, tab_view, i);
 /* !TAB_VIEW_GH_FOREACH_START */
 
 #define TAB_VIEW_GH_FOREACH_END												\
@@ -196,6 +211,41 @@ static void ghex_application_window_connect_hex_signals (GHexApplicationWindow *
 }																			\
 /* !TAB_VIEW_GH_FOREACH_END	*/
 
+static AdwTabPage *
+get_tab_for_gh (GHexApplicationWindow *self, HexWidget *target_gh)
+{
+	TAB_VIEW_GH_FOREACH_START
+
+	if (gh == target_gh)
+		return tab_page;
+
+	TAB_VIEW_GH_FOREACH_END
+
+	return NULL;
+}
+
+static HexInfoBar *
+get_info_bar_for_gh (GHexApplicationWindow *self, HexWidget *target_gh)
+{
+	TAB_VIEW_GH_FOREACH_START
+
+	if (gh == target_gh)
+	{
+		GtkWidget *box = adw_tab_page_get_child (tab_page);
+		for (GtkWidget *child = gtk_widget_get_first_child (box);
+				child != NULL;
+				child = gtk_widget_get_next_sibling (child))
+		{
+			if (HEX_IS_INFO_BAR (child))
+				return HEX_INFO_BAR(child);
+		}
+	}
+
+	TAB_VIEW_GH_FOREACH_END
+
+	return NULL;
+}
+
 /* set_*_from_settings
  * These functions all basically set properties from the GSettings global
  * variables. They should be used when a new gh is created, and when we're
@@ -257,6 +307,20 @@ settings_group_type_changed_cb (GSettings   *settings,
 	TAB_VIEW_GH_FOREACH_END
 }
 
+static void
+settings_show_control_chars_changed_cb (GSettings   *settings,
+		const gchar	*key,
+		gpointer 	user_data)
+{
+	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
+
+	TAB_VIEW_GH_FOREACH_START
+
+	hex_widget_set_display_control_characters (gh, def_display_control_characters);
+
+	TAB_VIEW_GH_FOREACH_END
+}
+
 /* ! settings*changed_cb 's */
 
 static void
@@ -316,37 +380,37 @@ file_save (GHexApplicationWindow *self)
 }
 
 static void
-close_all_tabs_response_cb (AdwMessageDialog *dialog,
+close_all_tabs_response_cb (AdwAlertDialog *dialog,
 		const char *response,
 		GHexApplicationWindow *self)
 {
 	/* Regardless of what the user chose, get rid of the dialog. */
-	gtk_window_close (GTK_WINDOW(dialog));
+	adw_dialog_close (ADW_DIALOG (dialog));
 
-	if (g_strcmp0 (response, "accept") == 0)
+	if (g_strcmp0 (response, "discard") == 0)
 		gtk_window_destroy (GTK_WINDOW(self));
 }
 
 static void
 close_all_tabs_confirmation_dialog (GHexApplicationWindow *self)
 {
-	GtkWidget *dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
+	AdwDialog *dialog = adw_alert_dialog_new (_("Save Changes?"), NULL);
 
-	adw_message_dialog_format_body_markup (ADW_MESSAGE_DIALOG(dialog),
-			_("<b>You have one or more files open with unsaved changes.</b>\n\n"
-			   "Are you sure you want to close the window?\n\n"));
-	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
-			"accept", _("_Close Anyway"),
-			"reject", _("_Go Back"),
+	adw_alert_dialog_set_body (ADW_ALERT_DIALOG(dialog),
+			_("Open documents contain unsaved changes.\n"
+			   "Changes which are not saved will be permanently lost."));
+	adw_alert_dialog_add_responses (ADW_ALERT_DIALOG(dialog),
+			"cancel", _("_Cancel"),
+			"discard", _("_Discard"),
 			NULL);
-	/* workaround since AdwMessageDialog sets destroy-with-parent to true, which
-	 * can cause runtime errors upon exit. See: https://discourse.gnome.org/t/glib-gobject-critical-and-glib-gobject-warning-messages/9909
-	 */
-	gtk_window_set_destroy_with_parent (GTK_WINDOW(dialog), FALSE);
+	adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG(dialog),
+			"discard",
+			ADW_RESPONSE_DESTRUCTIVE);
+	adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG(dialog), "cancel");
 
 	g_signal_connect (dialog, "response", G_CALLBACK(close_all_tabs_response_cb), self);
 
-	gtk_window_present (GTK_WINDOW(dialog));
+	adw_dialog_present (dialog, GTK_WIDGET(self));
 }
 
 static void
@@ -363,7 +427,7 @@ check_close_window (GHexApplicationWindow *self)
 		HexWidget *gh;
 		HexDocument *doc = NULL;
 
-		gh = GH_FOR_TAB (tab_view, i);
+		gh = get_gh_for_tab (self, tab_view, i);
 		doc = hex_widget_get_document (gh);
 
 		if (hex_document_has_changed (doc))
@@ -408,19 +472,19 @@ close_page_finish_helper (GHexApplicationWindow *self, AdwTabView *tab_view, Adw
 }
 
 static void
-close_doc_response_cb (AdwMessageDialog *dialog,
+close_doc_response_cb (AdwAlertDialog *dialog,
 		const char *response,
 		GHexApplicationWindow *self)
 {
 	AdwTabView *tab_view = ADW_TAB_VIEW(self->hex_tab_view);
 	AdwTabPage *page = g_object_get_data (G_OBJECT(self), "target-page");
-	
-	if (g_strcmp0 (response, "accept") == 0)
+
+	if (g_strcmp0 (response, "save") == 0)
 	{
 		file_save (self);
 		close_page_finish_helper (self, tab_view, page, TRUE);
 	}
-	else if (g_strcmp0 (response, "reject") == 0)
+	else if (g_strcmp0 (response, "discard") == 0)
 	{
 		close_page_finish_helper (self, tab_view, page, TRUE);
 	}
@@ -429,19 +493,20 @@ close_doc_response_cb (AdwMessageDialog *dialog,
 		close_page_finish_helper (self, tab_view, page, FALSE);
 	}
 
-	gtk_window_destroy (GTK_WINDOW(dialog));
+	adw_dialog_close (ADW_DIALOG (dialog));
 	gtk_widget_grab_focus (GTK_WIDGET (ACTIVE_GH));
 }
 
 static void
 close_doc_confirmation_dialog (GHexApplicationWindow *self, AdwTabPage *page)
 {
-	GtkWidget *dialog;
+	AdwDialog *dialog;
 	HexDocument *doc;
 	GFile *file;
+	char *title;
 	char *message;
 	char *basename = NULL;
-	HexWidget *gh = HEX_WIDGET(adw_tab_page_get_child (page));
+	HexWidget *gh = get_gh_for_page (self, page);
 	doc = hex_widget_get_document (gh);
 
 	g_return_if_fail (HEX_IS_DOCUMENT (doc));
@@ -450,36 +515,36 @@ close_doc_confirmation_dialog (GHexApplicationWindow *self, AdwTabPage *page)
 		basename = g_file_get_basename (hex_document_get_file (doc));
 
 	if (basename) {
-		message = g_strdup_printf (
-				/* Translators: %s is the filename that is currently being
-				 * edited. */
-				_("<big><b>%s has been edited since opening.</b></big>\n\n"
-			   "Would you like to save your changes?"), basename);
+		/* Translators: %s is the filename that is currently being
+		 * edited. */
+		title = g_strdup_printf (_("%s has been edited since opening."), basename);
 		g_free (basename);
 	}
 	else {
-		message = g_strdup (
-				_("<b>The buffer has been edited since opening.</b>\n\n"
-			   "Would you like to save your changes?"));
+		title = _("The buffer has been edited since opening.");
 	}
 
-	dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
-	adw_message_dialog_set_body_use_markup (ADW_MESSAGE_DIALOG(dialog), TRUE);
-	adw_message_dialog_set_body (ADW_MESSAGE_DIALOG(dialog), message);
+	message = _("Would you like to save your changes?");
 
-	g_free (message);
-
-	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
-			"accept", _("_Save Changes"),
-			"reject", _("_Discard Changes"),
-			"cancel", _("_Go Back"),
+	dialog = adw_alert_dialog_new (title, NULL);
+	adw_alert_dialog_set_body (ADW_ALERT_DIALOG(dialog), message);
+	adw_alert_dialog_add_responses (ADW_ALERT_DIALOG(dialog),
+			"cancel", _("_Cancel"),
+			"discard", _("_Discard"),
+			"save", _("_Save"),
 			NULL);
-	adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG(dialog), "cancel");
+	adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG(dialog), "cancel");
+	adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG(dialog),
+			"discard",
+			ADW_RESPONSE_DESTRUCTIVE);
+	adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG(dialog),
+			"save",
+			ADW_RESPONSE_SUGGESTED);
 	g_signal_connect (dialog, "response", G_CALLBACK(close_doc_response_cb), self);
 
 	g_object_set_data (G_OBJECT(self), "target-page", page);
 
-	gtk_window_present (GTK_WINDOW(dialog));
+	adw_dialog_present (ADW_DIALOG(dialog), GTK_WIDGET (self));
 }
 
 static void
@@ -534,7 +599,7 @@ copy_special (GtkWidget *widget,
 	gtk_window_present (GTK_WINDOW(self->copy_special_dialog));
 }
 
-static void 
+static void
 paste_special (GtkWidget *widget,
 		const char *action_name,
 		GVariant *parameter)
@@ -601,7 +666,7 @@ update_tabs (GHexApplicationWindow *self)
 	else
 		basename = g_strdup (_(UNTITLED_STRING));
 
-	adw_tab_page_set_title (TAB_FOR_GH (gh), basename);
+	adw_tab_page_set_title (get_tab_for_gh (self, gh), basename);
 	g_free (basename);
 
 	TAB_VIEW_GH_FOREACH_END
@@ -634,12 +699,13 @@ file_loaded (HexDocument *doc, GHexApplicationWindow *self)
 {
 	document_loaded_or_saved_common (self, doc);
 	update_gui_data (self);
-	adw_tab_page_set_icon (TAB_FOR_GH (ACTIVE_GH), NULL);
+	adw_tab_page_set_icon (get_tab_for_gh (self, ACTIVE_GH), NULL);
+	hex_info_bar_set_shown (get_info_bar_for_gh (self, ACTIVE_GH), FALSE);
 }
 
 static void
 document_changed_cb (HexDocument *doc,
-		gpointer change_data,
+		HexChangeData *change_data,
 		gboolean push_undo,
 		gpointer user_data)
 {
@@ -651,9 +717,15 @@ document_changed_cb (HexDocument *doc,
 
 	if (hex_widget_get_document (gh) == doc)
 	{
-		GIcon *icon = g_themed_icon_new ("document-modified-symbolic");
-		adw_tab_page_set_icon (TAB_FOR_GH (gh), icon);
+		GIcon *icon;
+		HexInfoBar *info_bar;
+
+		icon = g_themed_icon_new ("document-modified-symbolic");
+		adw_tab_page_set_icon (get_tab_for_gh (self, gh), icon);
 		g_object_unref (icon);
+
+		info_bar = get_info_bar_for_gh (self, gh);
+		hex_info_bar_set_shown (info_bar, change_data->external_file_change);
 	}
 
 	TAB_VIEW_GH_FOREACH_END
@@ -668,7 +740,7 @@ tab_view_close_page_cb (AdwTabView *tab_view,
 	HexDocument *doc;
 	HexWidget *gh;
 
-	gh = HEX_WIDGET(adw_tab_page_get_child (page));
+	gh = get_gh_for_page (self, page);
 	doc = hex_widget_get_document (gh);
 
 	if (hex_document_has_changed (doc)) {
@@ -706,9 +778,8 @@ tab_view_page_attached_cb (AdwTabView *tab_view,
 		gpointer user_data)
 {
 	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
-	
-	ghex_application_window_connect_hex_signals (self,
-			HEX_WIDGET(adw_tab_page_get_child (page)));
+
+	ghex_application_window_connect_hex_signals (self, get_gh_for_page (self, page));
 
 	/* Let's play this super dumb. If a page is added, that will generally
 	 * mean we don't have to count the pages to see if we have > 0.
@@ -724,8 +795,7 @@ tab_view_page_detached_cb (AdwTabView *tab_view,
 {
 	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(user_data);
 
-	ghex_application_window_disconnect_hex_signals (self,
-			HEX_WIDGET(adw_tab_page_get_child (page)));
+	ghex_application_window_disconnect_hex_signals (self, get_gh_for_page (self, page));
 }
 
 static AdwTabView *
@@ -1121,18 +1191,19 @@ save_as (GtkWidget *widget,
 	g_signal_connect (file_sel, "response",
 			G_CALLBACK(save_as_response_cb), self);
 
+	gtk_native_dialog_set_modal (GTK_NATIVE_DIALOG (file_sel), TRUE);
 	gtk_native_dialog_show (GTK_NATIVE_DIALOG(file_sel));
 }
 G_GNUC_END_IGNORE_DEPRECATIONS
 
 static void
-revert_response_cb (AdwMessageDialog *dialog,
+revert_response_cb (AdwAlertDialog *dialog,
 		const char *response,
 		GHexApplicationWindow *self)
 {
 	HexDocument *doc;
 
-	if (g_strcmp0 (response, "accept") != 0)
+	if (g_strcmp0 (response, "revert") != 0)
 		goto end;
 
 	doc = hex_widget_get_document (ACTIVE_GH);
@@ -1141,7 +1212,7 @@ revert_response_cb (AdwMessageDialog *dialog,
 	hex_document_read_async (doc, NULL, doc_read_ready_cb, self);
 
 end:
-	gtk_window_destroy (GTK_WINDOW(dialog));
+	adw_dialog_close (ADW_DIALOG(dialog));
 }
 
 static void
@@ -1151,10 +1222,10 @@ revert (GtkWidget *widget,
 {
 	GHexApplicationWindow *self = GHEX_APPLICATION_WINDOW(widget);
    	HexDocument *doc;
-	GtkWidget *dialog;
+	AdwDialog *dialog;
 	gint reply;
 	char *basename = NULL;
-	
+
 	g_return_if_fail (HEX_IS_WIDGET (ACTIVE_GH));
 
 	doc = hex_widget_get_document (ACTIVE_GH);
@@ -1166,27 +1237,28 @@ revert (GtkWidget *widget,
 
 	basename = g_file_get_basename (hex_document_get_file (doc));
 
-	dialog = adw_message_dialog_new (GTK_WINDOW(self), NULL, NULL);
-	adw_message_dialog_format_body_markup (ADW_MESSAGE_DIALOG(dialog),
-			/* Translators: %s here is the filename the user is being asked to
-			 * confirm whether they want to revert. */
-			_("<big><b>Are you sure you want to revert %s?</b></big>\n\n"
-			"Your changes will be lost.\n\n"
-			"This action cannot be undone."),
-			basename);
-	adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG(dialog),
-			"accept", _("_Revert"),
-			"reject", _("_Go Back"),
+	/* Translators: %s here is the filename the user is being asked to
+	 * confirm whether they want to revert. */
+	dialog = adw_alert_dialog_new (g_strdup_printf (_("Are you sure you want to revert %s?"), basename), NULL);
+	adw_alert_dialog_set_body (ADW_ALERT_DIALOG(dialog),
+			_("Your changes will be lost.\n"
+			"This action cannot be undone."));
+	adw_alert_dialog_add_responses (ADW_ALERT_DIALOG(dialog),
+			"cancel", _("_Cancel"),
+			"revert", _("_Revert"),
 			NULL);
-	adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG(dialog), "reject");
+	adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG(dialog), "cancel");
+	adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG(dialog),
+			"revert",
+			ADW_RESPONSE_DESTRUCTIVE);
 
 	g_signal_connect (dialog, "response", G_CALLBACK(revert_response_cb), self);
 
-	gtk_widget_set_visible (dialog, TRUE);
+	adw_dialog_present (dialog, GTK_WIDGET (self));
 
 	g_free (basename);
 }
-			
+
 static void
 print_preview (GtkWidget *widget,
 		const char *action_name,
@@ -1272,6 +1344,7 @@ open_file_action (GtkWidget *widget,
 	g_signal_connect (file_sel, "response",
 			G_CALLBACK(open_response_cb), self);
 
+	gtk_native_dialog_set_modal (GTK_NATIVE_DIALOG (file_sel), TRUE);
 	gtk_native_dialog_show (GTK_NATIVE_DIALOG(file_sel));
 }
 G_GNUC_END_IGNORE_DEPRECATIONS
@@ -1555,26 +1628,52 @@ update_status_message (GHexApplicationWindow *self)
 	char *status = NULL;
 	gint64 current_pos, ss, se;
 	int len;
+	gboolean selection;
 
 	if (! ACTIVE_GH)
 		goto out;
 
 	current_pos = hex_widget_get_cursor (ACTIVE_GH);
+	selection = hex_widget_get_selection (ACTIVE_GH, &ss, &se);
 
-	if (hex_widget_get_selection (ACTIVE_GH, &ss, &se))
-	{
-		status = g_strdup_printf (
-				_("Offset: <tt>0x%lX</tt>; <tt>0x%lX</tt> bytes from <tt>0x%lX</tt> to <tt>0x%lX</tt> selected"),
-				current_pos, se - ss + 1, ss, se);
-	}
-	else {
-		status = g_strdup_printf (_("Offset: <tt>0x%lX</tt>"), current_pos);
+	switch (def_sb_offset_format) {
+		case HEX_WIDGET_STATUS_BAR_OFFSET_HEX:
+			if (selection) {
+				status = g_strdup_printf (
+					_("Offset: <tt>0x%lX</tt>; <tt>0x%lX</tt> bytes from <tt>0x%lX</tt> to <tt>0x%lX</tt> selected"),
+					current_pos, se - ss + 1, ss, se);
+			} else {
+				status = g_strdup_printf (_("Offset: <tt>0x%lX</tt>"), current_pos);
+			}
+			break;
+
+		case HEX_WIDGET_STATUS_BAR_OFFSET_DEC:
+			if (selection) {
+				status = g_strdup_printf (
+					_("Offset: <tt>%ld</tt>; <tt>%ld</tt> bytes from <tt>%ld</tt> to <tt>%ld</tt> selected"),
+					current_pos, se - ss + 1, ss, se);
+			} else {
+				status = g_strdup_printf (_("Offset: <tt>%ld</tt>"), current_pos);
+			}
+			break;
+
+		case HEX_WIDGET_STATUS_BAR_OFFSET_HEX | HEX_WIDGET_STATUS_BAR_OFFSET_DEC:
+			if (selection) {
+				status = g_strdup_printf (
+					/* Weird rendering if you don't put the space at the start */
+					_(" <sub>HEX</sub> Offset: <tt>0x%lX</tt>; <tt>0x%lX</tt> bytes from <tt>0x%lX</tt> to <tt>0x%lX</tt> selected <sub>DEC</sub> Offset: <tt>%ld</tt>; <tt>%ld</tt> bytes from <tt>%ld</tt> to <tt>%ld</tt> selected"),
+					current_pos, se - ss + 1, ss, se,
+					current_pos, se - ss + 1, ss, se);
+			} else {
+				status = g_strdup_printf (_(" <sub>HEX</sub> Offset: <tt>0x%lX</tt> <sub>DEC</sub> Offset: <tt>%ld</tt>"), current_pos, current_pos);
+			}
+			break;
 	}
 
 	hex_statusbar_set_status (HEX_STATUSBAR(self->statusbar), status);
 	g_free (status);
 	return;
-	
+
 out:
 	hex_statusbar_clear (HEX_STATUSBAR(self->statusbar));
 }
@@ -1718,10 +1817,11 @@ ghex_application_window_init (GHexApplicationWindow *self)
 	GAction *action;
 	GtkDropTarget *target;
 	GtkCssProvider *provider;
+	GtkWindowGroup *group;
 
-	/* Ensure the custom statusbar widget is registered with the type system.
-	 */
+	/* Ensure widgets are registered with the type system. */
 	g_type_ensure (HEX_TYPE_STATUSBAR);
+	g_type_ensure (HEX_TYPE_INFO_BAR);
 
 	gtk_widget_init_template (widget);
 
@@ -1747,6 +1847,12 @@ ghex_application_window_init (GHexApplicationWindow *self)
     g_signal_connect (settings, "changed::" GHEX_PREF_GROUP,
                       G_CALLBACK (settings_group_type_changed_cb), self);
 
+    g_signal_connect (settings, "changed::" GHEX_PREF_CONTROL_CHARS,
+                      G_CALLBACK (settings_show_control_chars_changed_cb), self);
+
+    g_signal_connect_swapped (settings, "changed::" GHEX_PREF_SB_OFFSET_FORMAT,
+                      G_CALLBACK (update_status_message), self);
+
 	/* Actions - SETTINGS */
 
 	/* for the 'group data by' stuff. There isn't a function to do this from
@@ -1833,6 +1939,12 @@ ghex_application_window_init (GHexApplicationWindow *self)
 	gtk_style_context_add_provider_for_display (gdk_display_get_default (),
 			GTK_STYLE_PROVIDER (provider),
 			GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+	/* Add to window group */
+
+	group = gtk_window_group_new ();
+	gtk_window_group_add_window (group, GTK_WINDOW(self));
+	g_object_unref (group);
 }
 
 static void
@@ -1906,7 +2018,7 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			"Whether the Save (or Revert) button should currently be clickable",
 			FALSE,	/* gboolean default_value */
 			prop_flags);
-	
+
 	properties[PROP_INSERT_MODE] =
 		g_param_spec_boolean ("insert-mode",
 			"Insert mode",
@@ -2071,10 +2183,17 @@ ghex_application_window_class_init (GHexApplicationWindowClass *klass)
 			"ghex.copy-special",
 			NULL);
 
-	/* Ctrl+N - new */
+	/* Ctrl+N - new window */
 	gtk_widget_class_add_binding_action (widget_class,
 			GDK_KEY_n,
 			GDK_CONTROL_MASK,
+			"app.new-window",
+			NULL);
+
+	/* Ctrl+T - new tab */
+	gtk_widget_class_add_binding_action (widget_class,
+			GDK_KEY_t,
+			GDK_CONTROL_MASK,
 			"ghex.new",
 			NULL);
 
@@ -2285,8 +2404,7 @@ ghex_application_window_activate_tab (GHexApplicationWindow *self,
 
 	g_return_if_fail (HEX_IS_WIDGET (gh));
 
-	adw_tab_view_set_selected_page (tab_view,
-			adw_tab_view_get_page (tab_view, GTK_WIDGET(gh)));
+	adw_tab_view_set_selected_page (tab_view, get_tab_for_gh (self, gh));
 
 	gtk_widget_grab_focus (GTK_WIDGET(gh));
 }
@@ -2324,6 +2442,8 @@ void
 ghex_application_window_add_hex (GHexApplicationWindow *self,
 		HexWidget *gh)
 {
+	GtkWidget *info_bar;
+	GtkWidget *box;
 	GtkWidget *tab;
 	HexDocument *doc;
 
@@ -2338,12 +2458,22 @@ ghex_application_window_add_hex (GHexApplicationWindow *self,
 	/* HexWidget: Sync up appwindow-specific settings. */
 	set_gtkhex_offsets_column_from_settings (gh);
 	set_gtkhex_group_type_from_settings (gh);
-	
+	hex_widget_set_display_control_characters (gh, def_display_control_characters);
+
 	/* HexWidget: Set insert mode based on our global appwindow prop */
 	hex_widget_set_insert_mode (gh, self->insert_mode);
 
+	/* Setup box to hold info_bar and HexWidget */
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	info_bar = hex_info_bar_new ();
+
+	gtk_box_append (GTK_BOX(box), info_bar);
+	gtk_box_append (GTK_BOX(box), GTK_WIDGET(gh));
+	gtk_widget_set_hexpand (GTK_WIDGET(gh), TRUE);
+	gtk_widget_set_vexpand (GTK_WIDGET(gh), TRUE);
+
 	/* Add tab */
-	adw_tab_view_append (ADW_TAB_VIEW(self->hex_tab_view), GTK_WIDGET(gh));
+	adw_tab_view_append (ADW_TAB_VIEW(self->hex_tab_view), box);
 
 	show_hex_tab_view (self);
 }
@@ -2386,7 +2516,7 @@ do_nag_screen (GHexApplicationWindow *self)
 				"this GHex session.\n\n"
 				"To avoid this message from appearing, try using a different "
 				"buffer backend.");
-	
+
 		g_printerr ("%s", msg);
 		g_printerr ("\n");
 
@@ -2440,7 +2570,7 @@ doc_read_ready_cb (GObject *source_object,
 		}
 	}
 }
-			
+
 void
 ghex_application_window_open_file (GHexApplicationWindow *self, GFile *file)
 {
@@ -2524,7 +2654,17 @@ ghex_application_window_get_hex (GHexApplicationWindow *self)
 	page = adw_tab_view_get_selected_page (ADW_TAB_VIEW(self->hex_tab_view));
 
 	if (page)
-		return HEX_WIDGET(adw_tab_page_get_child (page));
+	{
+		GtkWidget *box = adw_tab_page_get_child (page);
+		for (GtkWidget *child = gtk_widget_get_first_child (box);
+				child != NULL;
+				child = gtk_widget_get_next_sibling (child))
+		{
+			if (HEX_IS_WIDGET (child))
+				return HEX_WIDGET(child);
+		}
+		return NULL;
+	}
 	else
 		return NULL;
 }
