@@ -36,7 +36,13 @@
 
 #include <config.h>
 
-GOptionEntry entries[] = {
+static GOptionEntry entries[] = {
+	{	.long_name = 	"new-window",
+		.short_name =	'w',
+		.flags = 		G_OPTION_FLAG_NONE,
+		.arg =			G_OPTION_ARG_NONE,
+		.description =	N_("Open a new window"),
+	},
 	{	.long_name = 	"version",
 		.short_name =	'v',
 		.flags = 		G_OPTION_FLAG_NONE,
@@ -46,8 +52,6 @@ GOptionEntry entries[] = {
 	{ NULL }
 };
 
-static GtkWindow *window = NULL;
-
 /* FIXME - TEST ON WIN32.
  * This is one of the few functions in this file that has been ripped verbatim
  * from the old main.c. It might work. Maybe.
@@ -85,21 +89,21 @@ ghex_locale_dir (void)
 #endif
 }
 
-static void
-do_app_window (AdwApplication *app)
-{
-	if (! window)
-		window = GTK_WINDOW(ghex_application_window_new (app));
-	else
-		g_return_if_fail (GHEX_IS_APPLICATION_WINDOW
-				(GHEX_APPLICATION_WINDOW(window)));
-}
-
 static int
 handle_local_options (GApplication *application,
 		GVariantDict *options,
 		gpointer      user_data)
 {
+	if (g_variant_dict_contains (options, "new-window"))
+	{
+		g_application_register (application, NULL, NULL);
+
+		if (g_application_get_is_remote (application))
+		{
+			g_action_group_activate_action (G_ACTION_GROUP(application), "new-window", NULL);
+			return 0;
+		}
+	}
 	if (g_variant_dict_contains (options, "version"))
 	{
 		g_print (_("This is GHex, version %s\n"), PACKAGE_VERSION);
@@ -109,17 +113,31 @@ handle_local_options (GApplication *application,
 	return -1;		/* let processing continue (see TFM) */
 }
 
+static void
+startup (AdwApplication *app,
+	gpointer user_data)
+{
+	ghex_init_configuration ();
+
+	/* WORKAROUND https://gitlab.gnome.org/GNOME/gtk/-/issues/4880 */
+	g_object_set (gtk_settings_get_default (), "gtk-dialogs-use-header", TRUE, NULL);
+
+}
+
 static void
 activate (AdwApplication *app,
 	gpointer user_data)
 {
-	/* WORKAROUND https://gitlab.gnome.org/GNOME/gtk/-/issues/4880 */
+	GtkWindow *window;
 
-	g_object_set (gtk_settings_get_default (), "gtk-dialogs-use-header", TRUE, NULL);
+	window = gtk_application_get_active_window (GTK_APPLICATION(app));
+	if (window)
+	{
+		gtk_window_present (window);
+		return;
+	}
 
-	do_app_window (app);
-
-	gtk_window_set_application (window, GTK_APPLICATION(app));
+	window = GTK_WINDOW(ghex_application_window_new (app));
 	gtk_window_present (window);
 }
 
@@ -130,19 +148,31 @@ open (GApplication *application,
 		const char *hint,
 		gpointer user_data)
 {
-	GHexApplicationWindow *app_win;
+	GtkWindow *window;
 
 	activate (ADW_APPLICATION(application), NULL);
-	app_win = GHEX_APPLICATION_WINDOW(window);
+	window = gtk_application_get_active_window (GTK_APPLICATION(application));
 
 	for (int i = 0; i < n_files; ++i)
-		ghex_application_window_open_file (app_win, files[i]);
+		ghex_application_window_open_file (GHEX_APPLICATION_WINDOW(window), files[i]);
+}
+
+static void
+new_window (GAction *action,
+		GVariant *parameter,
+		AdwApplication *app)
+{
+	GtkWindow *window;
+
+	window = GTK_WINDOW(ghex_application_window_new (app));
+	gtk_window_present (window);
 }
 
 int
 main (int argc, char *argv[])
 {
 	AdwApplication *app;
+	GSimpleAction *new_window_action;
 	char *locale_dir;
 	int status;
 
@@ -156,8 +186,6 @@ main (int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 	/* </i18n> */
 
-	ghex_init_configuration ();
-
 	app = adw_application_new (APP_ID, G_APPLICATION_HANDLES_OPEN);
 
 	g_application_add_main_option_entries (G_APPLICATION(app), entries);
@@ -165,12 +193,15 @@ main (int argc, char *argv[])
 	g_application_set_option_context_summary (G_APPLICATION(app),
 			_("GHex - A hex editor for the GNOME desktop"));;
 
+	g_signal_connect (app, "startup", G_CALLBACK(startup), NULL);
 	g_signal_connect (app, "activate", G_CALLBACK(activate), NULL);
 	g_signal_connect (app, "open", G_CALLBACK(open), NULL);
 	g_signal_connect (app, "handle-local-options",
 			G_CALLBACK(handle_local_options), NULL);
 
-	g_application_register (G_APPLICATION (app), NULL, NULL);
+	new_window_action = g_simple_action_new ("new-window", NULL);
+	g_signal_connect (new_window_action, "activate", G_CALLBACK(new_window), app);
+	g_action_map_add_action (G_ACTION_MAP(app), G_ACTION(new_window_action));
 
 	status = g_application_run (G_APPLICATION(app), argc, argv);
 
